// Copyright (C) 2002, 2003, NLnet Labs, All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <config.h>

#include "pcap.hh"

#include <algorithm>
#include <cassert>
#include <cstring>

namespace dns_analyzer {

    packet_capture::packet_capture() : _good(true), _pcap(0) {
        std::fill_n(_errbuf, PCAP_ERRBUF_SIZE, '\0');
    }

    packet_capture::~packet_capture() {
        close();
    }

    void packet_capture::open_offline(string const &filename) {
        _pcap = pcap_open_offline(filename.c_str(), _errbuf);
        _good = _pcap != 0;
    }

    void packet_capture::close() {
        if (_pcap) {
            pcap_close(_pcap);
            _pcap = 0;
        }
        _good = true;
    }

    u_char const *packet_capture::next(pcap_pkthdr *h) {
        return pcap_next(_pcap, h);
    }

    bool packet_capture::good() const {
        return _good;
    }

    char const *packet_capture::error() const {
        return _good ? 0 : _errbuf;
    }

    packet_dumper_type::packet_dumper_type(packet_capture &capture)
        : _capture(capture), _good(true), _pcap_dumper(0)
    {
        std::fill_n(_errbuf, PCAP_ERRBUF_SIZE, '\0');
    }

    packet_dumper_type::~packet_dumper_type() {
        close();
    }

    void
    packet_dumper_type::open(string const &filename) {
        assert(_capture._pcap);
        _pcap_dumper = pcap_dump_open(_capture._pcap, filename.c_str());
        _good = _pcap_dumper != 0;
        if (!_good) {
            strcpy(_errbuf, pcap_geterr(_capture._pcap));
        }
    }

    void
    packet_dumper_type::close() {
        if (_pcap_dumper) {
            pcap_dump_close(_pcap_dumper);
            _pcap_dumper = 0;
        }
        _good = true;
    }

    void
    packet_dumper_type::dump(pcap_pkthdr *header, u_char const *packet) {
        if (_pcap_dumper) {
            // It would be nice if stuff like this was documented in
            // the pcap manual page.
            pcap_dump(reinterpret_cast<u_char *>(_pcap_dumper), header, packet);
        }
    }

    bool
    packet_dumper_type::good() const {
        return _good;
    }

    char
    const *packet_dumper_type::error() const {
        return _good ? 0 : _errbuf;
    }

};
