// Copyright (C) 2002, 2003, NLnet Labs, All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef PCAP_HH
#define PCAP_HH

#include <string>
extern "C" {
#include <pcap.h>
}

namespace dns_analyzer {

    using std::string;

    class packet_dumper_type;

    class packet_capture {
    public:
        packet_capture();

        /**
         * Closes associated files and deallocates resources if
         * necessary.
         */
        ~packet_capture();

        /**
         * Open a savefile for reading.  The filename "-" is a synonym
         * for <code>stdin</code>.
         */
        void open_offline(string const &filename);

        /**
         * Closes associated files and deallocates resources.
         */
        void close();

        /**
         * Waits for and reads the next packet.
         *
         * @return The packet data.  This data is statically
         * allocated.
         */
        u_char const *next(pcap_pkthdr *h);

        /**
         * True iff no error has occurred.
         */
        bool good() const;

        /**
         * Non-null iff an error has occurred.  The result points to
         * the error message returned by the pcap library.
         */
        char const *error() const;

    private:
        bool _good;
        pcap_t *_pcap;
        char _errbuf[PCAP_ERRBUF_SIZE];

        /**
         * Prohibit copying.
         */
        packet_capture(packet_capture const &rhs);

        /**
         * Prohibit copying.
         */
        packet_capture const &operator=(packet_capture const &rhs);

        friend class packet_dumper_type;
    };

    class packet_dumper_type {
    public:
        explicit packet_dumper_type(packet_capture &capture);
        ~packet_dumper_type();

        /**
         * Opens a dump file for saving packets.
         */
        void open(string const &filename);

        /**
         * Saves the packet to the dump file.
         */
        void dump(pcap_pkthdr *header, u_char const *packet);

        /**
         * Closes the dump file.
         */
        void close();

        /**
         * True iff no error has occurred.
         */
        bool good() const;

        /**
         * Non-null iff an error has occurred.  The result points to
         * the error message returned by the pcap library.
         */
        char const *error() const;
    private:
        packet_capture &_capture;
        bool _good;
        pcap_dumper_t *_pcap_dumper;
        char _errbuf[PCAP_ERRBUF_SIZE];
    };
};

#endif // PCAP_HH
