// Copyright (C) 2002, 2003, NLnet Labs, All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <config.h>

#include "performance.hh"
#include <iostream>

namespace dns_analyzer {

    using std::cerr;

    performance_listener_type::performance_listener_type()
        : _timer(), _packet_count(0), _stats()
    {
    }

    performance_listener_type::~performance_listener_type() {
    }

    void
    performance_listener_type::initialize(analyzer_type &/*analyzer*/) {
        _timer.start();
    }

    void
    performance_listener_type::analyze(analyzer_type &/*analyzer*/,
                                       timeval const &/*timestamp*/,
                                       network_packet const &/*packet*/)
    {
        ++_packet_count;
        if (_packet_count % 10000 == 0) {
            _timer.stop();
            _stats.add(_packet_count / _timer.cpu_time());
        }
    }

    void
    performance_listener_type::finalize(analyzer_type &/*analyzer*/) {
        _timer.stop();
        _stats.add(_packet_count / _timer.cpu_time());
        cerr << "Analyzed " << _packet_count / _timer.cpu_time()
             << " packets per second.\n";
        cerr << "Stats: " << _stats << '\n';
    }
}
