// Copyright (C) 2002, 2003, NLnet Labs, All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//
// Small program to convert tcpdump(1) tracefiles to C code.
//

#include <config.h>

#include "../pcap.hh"
#include <iomanip>
#include <iostream>
#include <cstdlib>

int
main(int argc, char **argv) {

    using namespace dns_analyzer;
    using namespace std;

    if (argc != 2) {
        cerr << "Usage: " << argv[0] << " tracefile\n";
        return EXIT_FAILURE;
    }

    packet_capture capture;

    capture.open_offline(argv[1]);
    if (!capture.good()) {
        cerr << argv[0] << ": " << capture.error() << endl;
        return EXIT_FAILURE;
    }

    cout << "#include <pcap.h>\n";

    unsigned count = 0;
    pcap_pkthdr header;
    while (u_char const *data = capture.next(&header)) {
        count++;

        cout << "\nstruct pcap_pkthdr header_" << dec << count << " = {\n"
             << "    { " << header.ts.tv_sec << ", " << header.ts.tv_usec
             << " },\n"
             << "    " << header.caplen << ",\n"
             << "    " << header.len << "\n};\n"
             << "unsigned char packet_" << dec << count << "[] = {";
        for (unsigned i = 0; i < header.caplen; i++) {
            if (i % 8 == 0) {
                cout << "\n   ";
            }
            cout << " 0x" << hex << setfill('0') << setw(2) << (int) data[i];
            if (i < header.caplen - 1) {
                cout << ",";
            }
        }
        cout << "\n};\n";
    }

    capture.close();

    return EXIT_SUCCESS;
}
