// Copyright (C) 2002, 2003, NLnet Labs, All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <config.h>

#include "util.hh"

#include <ios>
#include <sstream>

namespace dns_analyzer {

    using std::ios;
    using std::ostringstream;

    double
    timeval_to_double(timeval const &tv) {
        return double(tv.tv_sec) + double(tv.tv_usec) / 1.0e6;
    }

    string
    quote_string(string const &s) {
        ostringstream result;
        write_quoted_string(result, s);
        return result.str();
    }

    ostream &
    write_quoted_string(ostream &out, string const &in) {
        out << '"';
        for (string::const_iterator it = in.begin(); it != in.end(); ++it) {
            switch (*it) {
            case '"': case '\\':
                out << '\\' << *it;
                break;
            case '\n':
                out << "\\n";
                break;
            default:
                if (!isprint(*it)) {
                    out << "\\x";
                    out.fill('0');
                    out.width(2);
                    out.setf(ios::hex, ios::basefield);
                    out << static_cast<unsigned>(static_cast<unsigned char>(*it));
                } else {
                    out << *it;
                }
                break;
            }
        }
        out << '"';
        return out;
    }
}
